@@ -5,11 +5,11 @@ use warnings;
 use strict;
 use IkiWiki;
 use Encode;
+use URI::Escape q{uri_escape_utf8};
 use open qw{:utf8 :std};
 
 my $sha1_pattern     = qr/[0-9a-fA-F]{40}/; # pattern to validate Git sha1sums
 my $dummy_commit_msg = 'dummy commit';      # message to skip in recent changes
-my $git_dir=undef;
 
 sub import {
 	hook(type => "checkconfig", id => "git", call => \&checkconfig);
@@ -151,6 +151,17 @@ sub genwrapper {
 	}
 }
 
+my $git_dir=undef;
+my $prefix=undef;
+
+sub in_git_dir ($$) {
+	$git_dir=shift;
+	my @ret=shift->();
+	$git_dir=undef;
+	$prefix=undef;
+	return @ret;
+}
+
 sub safe_git (&@) {
 	# Start a child process safely without resorting to /bin/sh.
 	# Returns command output (in list content) or success state
@@ -302,8 +313,6 @@ sub merge_past ($$$) {
 	return $conflict;
 }
 
-{
-my $prefix;
 sub decode_git_file ($) {
 	my $file=shift;
 
@@ -325,7 +334,6 @@ sub decode_git_file ($) {
 
 	return decode("utf8", $file);
 }
-}
 
 sub parse_diff_tree ($) {
 	# Parse the raw diff tree chunk and return the info hash.
@@ -333,8 +341,8 @@ sub parse_diff_tree ($) {
 	my $dt_ref = shift;
 
 	# End of stream?
-	return if !defined @{ $dt_ref } ||
-		  !defined @{ $dt_ref }[0] || !length @{ $dt_ref }[0];
+	return if ! @{ $dt_ref } ||
+		  !defined $dt_ref->[0] || !length $dt_ref->[0];
 
 	my %ci;
 	# Header line.
@@ -461,13 +469,10 @@ sub git_sha1 (;$) {
 	# Ignore error since a non-existing file might be given.
 	my ($sha1) = run_or_non('git', 'rev-list', '--max-count=1', 'HEAD',
 		'--', $file);
-	if ($sha1) {
+	if (defined $sha1) {
 		($sha1) = $sha1 =~ m/($sha1_pattern)/; # sha1 is untainted now
 	}
-	else {
-		debug("Empty sha1sum for '$file'.");
-	}
-	return defined $sha1 ? $sha1 : q{};
+	return defined $sha1 ? $sha1 : '';
 }
 
 sub rcs_update () {
@@ -610,9 +615,10 @@ sub rcs_recentchanges ($) {
 		my @pages;
 		foreach my $detail (@{ $ci->{'details'} }) {
 			my $file = $detail->{'file'};
+			my $efile = uri_escape_utf8($file);
 
 			my $diffurl = defined $config{'diffurl'} ? $config{'diffurl'} : "";
-			$diffurl =~ s/\[\[file\]\]/$file/go;
+			$diffurl =~ s/\[\[file\]\]/$efile/go;
 			$diffurl =~ s/\[\[sha1_parent\]\]/$ci->{'parent'}/go;
 			$diffurl =~ s/\[\[sha1_from\]\]/$detail->{'sha1_from'}/go;
 			$diffurl =~ s/\[\[sha1_to\]\]/$detail->{'sha1_to'}/go;
@@ -845,9 +851,9 @@ sub rcs_receive () {
 		# (Also, if a subdir is involved, we don't want to chdir to
 		# it and only see changes in it.)
 		# The pre-receive hook already puts us in the right place.
-		$git_dir=".";
-		push @rets, git_parse_changes(0, git_commit_info($oldrev."..".$newrev));
-		$git_dir=undef;
+		in_git_dir(".", sub {
+			push @rets, git_parse_changes(0, git_commit_info($oldrev."..".$newrev));
+		});
 	}
 
 	return reverse @rets;
@@ -860,23 +866,21 @@ sub rcs_preprevert ($) {
 	# Examine changes from root of git repo, not from any subdir,
 	# in order to see all changes.
 	my ($subdir, $rootdir) = git_find_root();
-	$git_dir=$rootdir;
+	in_git_dir($rootdir, sub {
+		my @commits=git_commit_info($sha1, 1);
+	
+		if (! @commits) {
+			error "unknown commit"; # just in case
+		}
 
-	my @commits=git_commit_info($sha1, 1);
-	if (! @commits) {
-		error "unknown commit"; # just in case
-	}
+		# git revert will fail on merge commits. Add a nice message.
+		if (exists $commits[0]->{parents} &&
+		    @{$commits[0]->{parents}} > 1) {
+			error gettext("you are not allowed to revert a merge");
+		}
 
-	# git revert will fail on merge commits. Add a nice message.
-	if (exists $commits[0]->{parents} &&
-	    @{$commits[0]->{parents}} > 1) {
-		error gettext("you are not allowed to revert a merge");
-	}
-
-	my @ret=git_parse_changes(1, @commits);
-
-	$git_dir=undef;
-	return @ret;
+		git_parse_changes(1, @commits);
+	});
 }
 
 sub rcs_revert ($) {
